{% load enterprise %}
<img class="enterprise-logo" src="{{ enterprise_customer.safe_branding_configuration.safe_logo_url }}" alt="{{ enterprise_customer.name }}"/>
{% if workflow_text_override_available %}
<p class="partnered-text">
  {{ workflow_left_sidebar_text|only_safe_html }}
</p>
{% elif text_override_available %}
  <p class="partnered-text">
    {{ left_sidebar_text|only_safe_html }}
  </p>
{% else %}
  <p class="partnered-text">
    {{ welcome_text }}
  </p>
  <p class="partnered-text">
    {{ enterprise_welcome_text|only_safe_html }}
  </p>
{% endif %}
