# Generated by Django 3.2.23 on 2024-09-04 14:35

from django.db import migrations
import fernet_fields.fields


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0020_remove_sapsuccessfactorsenterprisecustomerconfiguration_decrypted_secret'),
    ]

    operations = [
        migrations.AddField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='decrypted_secret',
            field=fernet_fields.fields.EncryptedCharField(blank=True, default='', help_text='The encrypted OAuth client secret. It will be encrypted when stored in the database.', max_length=255, null=True, verbose_name='Encrypted Client Secret'),
        ),
    ]
