# Generated by Django 4.2.21 on 2025-05-19 17:15

from django.db import migrations, connection


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0024_auto_20240909_1614'),
    ]

    database_operations = []
    if connection.vendor != 'sqlite':
        database_operations.append(
            migrations.RenameIndex(
                model_name='sapsuccessfactorslearnerdatatransmissionaudit',
                new_name='success_customer_plugin_idx',
                old_fields=('enterprise_customer_uuid', 'plugin_configuration_id'),
            ),
        )

    operations = [
        migrations.SeparateDatabaseAndState(
            database_operations=database_operations,
            state_operations=[
                migrations.RenameIndex(
                    model_name='sapsuccessfactorslearnerdatatransmissionaudit',
                    new_name='success_customer_plugin_idx',
                    old_fields=('enterprise_customer_uuid', 'plugin_configuration_id'),
                ),
            ],
        )
    ]
