# Generated by Django 4.2.17 on 2025-02-25 22:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0228_alter_defaultenterpriseenrollmentrealization_realized_enrollment'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomeruser',
            name='user_fk',
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomeruser',
            name='user_fk',
            field=models.IntegerField(null=True),
        ),
    ]
