# Generated by Django 2.2.14 on 2020-08-03 14:55

from django.db import migrations, models

import enterprise.validators


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0104_sync_query_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomerbrandingconfiguration',
            name='primary_color',
            field=models.CharField(blank=True, max_length=7, null=True, validators=[enterprise.validators.validate_hex_color]),
        ),
        migrations.AddField(
            model_name='enterprisecustomerbrandingconfiguration',
            name='secondary_color',
            field=models.CharField(blank=True, max_length=7, null=True, validators=[enterprise.validators.validate_hex_color]),
        ),
        migrations.AddField(
            model_name='enterprisecustomerbrandingconfiguration',
            name='tertiary_color',
            field=models.CharField(blank=True, max_length=7, null=True, validators=[enterprise.validators.validate_hex_color]),
        ),
    ]
