# Generated by Django 3.2.23 on 2024-09-04 14:20

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0019_auto_20240902_1418'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='decrypted_secret',
        ),
    ]
