# Generated by Django 4.2.21 on 2025-06-05 07:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0239_backfill_enable_learner_credit_message_box'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_data_sharing_consent',
            field=models.BooleanField(default=True, help_text='Enables data sharing consent prompt for learners each time they enroll in a course. If left unchecked, the prompt will not appear and relevant data will not be shared.', verbose_name='Activate data sharing consent prompt'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_data_sharing_consent',
            field=models.BooleanField(default=True, help_text='Enables data sharing consent prompt for learners each time they enroll in a course. If left unchecked, the prompt will not appear and relevant data will not be shared.', verbose_name='Activate data sharing consent prompt'),
        ),
    ]
